__author__ = 'andrew'

from acomms.modem_connections import ModemConnection
from threading import Thread
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.application import MIMEApplication
from email.mime.text import MIMEText
import email, os
from time import sleep
import imaplib
import datetime

class SBDEmailConnection(ModemConnection):
    def __init__(self, modem, IMEI,
                 email_account='acomms-sbd@whoi.edu',
                 username = None,pw = None,
                 check_rate_min = 5,
                 imap_srv = "imap.whoi.edu", imap_port = 143,
                 smtp_svr = "outbox.whoi.edu", smtp_port = 25, DoD=False):
        self.modem = modem
        self.SUBJECT = IMEI
        self.FROM = email_account
        self.email_username = username
        self.email_userpw = pw
        self.IMEI = IMEI
        self.email_check_rate = check_rate_min
        self.email_incoming_svr = imap_srv
        self.email_outgoing_svr = smtp_svr
        self.email_outgoing_port = smtp_port
        self.email_incoming_port = imap_port
        self.last_read = datetime.datetime.now()
        self.UseDoDEmail = DoD
        self.Alive = True
        self._threadL = Thread(target=self._listen)
        self._threadL.setDaemon(True)
        self._threadL.start()
        self._threadT = Thread(target=self._talk)
        self._threadT.setDaemon(True)
        self._threadT.start()


    @property
    def is_connected(self):
        return True

    @property
    def can_change_baudrate(self):
        return False

    def change_baudrate(self,baudrate):
        return 9600

    def _talk(self):
        while self.Alive:
            self.modem._process_outgoing_nmea()
            sleep(self.email_check_rate)

    def _listen(self):
        if self.UseDoDEmail:
            ArrivalEmail = "service@sbd.pac.disa.mil"
        else:
            ArrivalEmail = "sbdservice@sbd.iridium.com"
        while self.Alive:
            date = (datetime.datetime.now() - datetime.timedelta(1)).strftime("%d-%b-%Y")
            M = imaplib.IMAP4(self.email_incoming_svr, self.email_incoming_port)
            if self.email_username is not None:
                response, details = M.login(self.email_username, self.email_userpw)
            M.select('INBOX')
            #Limit our search to Unseen Messages for our IMEI in the Past 24 Hours from Iridium Only
            response, items = M.search(None,
                                       '(UNSEEN SENTSINCE {date} HEADER Subject "SBD Msg From Unit: {IMEI}" FROM "{Email}")'.format(
                                           date=date,
                                           IMEI=self.IMEI,
                                           Email = ArrivalEmail))
            for emailid in items[0].split():
                response, data = M.fetch(emailid, '(RFC822)')
                mail = email.message_from_string(data[0][1])

                if not mail.is_multipart():
                    continue
                for part in mail.walk():
                    if part.is_multipart():
                        continue
                    if part.get('Content-Disposition') is None:
                        continue
                    file_nm = part.get_filename()
                    if file_nm is None:
                        continue
                    filename, fileext = os.path.splitext(file_nm)
                    if fileext != '.sbd':
                        continue
                    msg = part.get_payload(decode=True)
                    for line in msg.splitlines(True):
                        self.modem._process_incoming_nmea(line)
                    temp = M.store(emailid,'+FLAGS', '\\Seen')
            M.close()
            M.logout()
            sleep(self.email_check_rate * 60) # Wait a minute and try again.

    def write(self,msg):
        email_msg = MIMEMultipart()
        email_msg['Subject'] = "{0}".format(self.IMEI)
        if self.UseDoDEmail:
            email_msg['To'] = 'data@sbd.pac.disa.mil'
        else:
            email_msg['To'] = 'data@sbd.iridium.com'
        email_msg['From'] = "{0}".format(self.FROM)

        part = MIMEText(msg)
        email_msg.attach(part)

        attachment = MIMEApplication(msg)

        attachment.add_header('Content-Disposition', 'attachment',
                                filename="{0}.sbd".format((datetime.datetime.utcnow()).strftime("%d%b%YT%H%M%S%z")))
        email_msg.attach(attachment)

        smtp = smtplib.SMTP()
        smtp.connect(self.email_outgoing_svr,  self.email_outgoing_port)
        smtp.sendmail(self.FROM,[email_msg['To'], self.FROM], email_msg.as_string())
        smtp.quit()

    def close(self):
        self.Alive = False


"""
This demo demonstrates how to embed a matplotlib (mpl) plot 
into a PyQt4 GUI application, including:

* Using the navigation toolbar
* Adding data to the plot
* Dynamically modifying the plot's properties
* Processing mpl events
* Saving the plot to a file from a menu

The main goal is to serve as a basis for developing rich PyQt GUI
applications featuring mpl plots (using the mpl OO API).

Eli Bendersky (eliben@gmail.com)
License: this code is in the public domain
Last modified: 19.01.2009
"""
import sys, os, random
from PySide.QtCore import *
from PySide.QtGui import *
from acomms import CycleStats, CycleStatsList
import magiccstbox



os.environ['QT_API'] = 'pyside'
import matplotlib
matplotlib.use('Qt4Agg')
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt4agg import NavigationToolbar2QTAgg as NavigationToolbar
from matplotlib.figure import Figure




class AppForm(QMainWindow):
    def __init__(self, cst_list, parent=None):
        QMainWindow.__init__(self, parent)
        self.setWindowTitle('Magic CST GUI')

        self.create_menu()
        self.create_main_frame()
        self.create_status_bar()

        self.textbox.setText('1 2 3 4')
        
        
        self.data = [1, 2, 3, 4]
        #self.data = [result.cst['snr_in'] for result in results_list]
        #self.data = cst_list;
        
        self.cst_list = cst_list
        self.cst_dol = cst_list.to_dict_of_lists()
        
        
              
        self.table.setColumnCount(len(cst_list[0]))
        self.table.setRowCount(len(cst_list))
        # Optional, set the labels that show on top
        self.table.setHorizontalHeaderLabels(CycleStats.fields)
        
        for row_num in range(len(cst_list)):
            for col_num in range(len(CycleStats.fields)):
                text = cst_list[row_num][CycleStats.fields[col_num]]
                table_item = QTableWidgetItem(str(text))
                self.table.setItem(row_num, col_num, table_item)

        # Also optional. Will fit the cells to its contents.
        self.table.resizeColumnsToContents()

        
 
        
        self.on_draw()
        
        

    def save_plot(self):
        file_choices = "PNG (*.png)|*.png"
        
        path = unicode(QFileDialog.getSaveFileName(self, 
                        'Save file', '', 
                        file_choices))
        if path:
            self.canvas.print_figure(path, dpi=self.dpi)
            self.statusBar().showMessage('Saved to %s' % path, 2000)
    
    def on_about(self):
        msg = """ The Magic CST GUI
        """
        QMessageBox.about(self, "About the demo", msg.strip())
    
    def on_pick(self, event):
        # The event received here is of the type
        # matplotlib.backend_bases.PickEvent
        #
        # It carries lots of information, of which we're using
        # only a small amount here.
        # 
        box_points = event.artist.get_bbox().get_points()
        msg = "You've clicked on a dot with coords:\n %s" % box_points
        
        QMessageBox.information(self, "Click!", msg)
    
    def on_draw(self):
        """ Redraws the figure
        """
        #str = unicode(self.textbox.text())
        #self.data = map(int, str.split())
        
        xdata = self.cst_dol['toa']

        # clear the axes and redraw the plot anew
        #
        
        plot_fields = []
        # Create subplots for each checked item
        for index in xrange(self.field_listbox.count()):
            item = self.field_listbox.item(index)
            ischecked = bool(item.checkState())
            if ischecked:
                plot_fields.append(item.text())
        
        if len(plot_fields) > 0:
            nrows = len(plot_fields)
            ncols = 1
            
            self.fig.clear()
            for row in xrange(nrows):
                ydata = self.cst_dol[plot_fields[row]]
                axes = self.fig.add_subplot(nrows, ncols, row)
                axes.plot(xdata, ydata, marker=".", linewidth=1)
                axes.set_ylabel(plot_fields[row])

        
        else:        
            
            # Pick the data based on the combo box value
            self.axes = self.fig.add_subplot('111')
            ydata = self.cst_dol[str(self.plot_combobox.currentText())]
            
            
            self.axes.clear()        
            self.axes.grid(self.grid_cb.isChecked())
                    
            self.axes.plot(xdata, ydata, marker=".", linewidth=self.slider.value())
        
        self.fig.autofmt_xdate()
        
        self.canvas.draw()
    
    def create_main_frame(self):
        self.main_frame = QWidget()
        
        # Create the mpl Figure and FigCanvas objects. 
        # 5x4 inches, 100 dots-per-inch
        #
        self.dpi = 100
        self.fig = Figure(dpi=self.dpi)
        self.canvas = FigureCanvas(self.fig)
        self.canvas.setParent(self.main_frame)
        
        # Since we have only one plot, we can use add_axes 
        # instead of add_subplot, but then the subplot
        # configuration tool in the navigation toolbar wouldn't
        # work.
        #
        
        
        # Bind the 'pick' event for clicking on one of the bars
        #
        self.canvas.mpl_connect('pick_event', self.on_pick)
        
        # Create the navigation toolbar, tied to the canvas
        #
        self.mpl_toolbar = NavigationToolbar(self.canvas, self.main_frame)
        
        # Other GUI controls
        # 
        self.textbox = QLineEdit()
        self.textbox.setMinimumWidth(200)
        self.connect(self.textbox, SIGNAL('editingFinished ()'), self.on_draw)
        
        self.draw_button = QPushButton("&Draw")
        self.connect(self.draw_button, SIGNAL('clicked()'), self.on_draw)
        
        self.grid_cb = QCheckBox("Show &Grid")
        self.grid_cb.setChecked(False)
        self.connect(self.grid_cb, SIGNAL('stateChanged(int)'), self.on_draw)
        
        slider_label = QLabel('Line width:')
        self.slider = QSlider(Qt.Horizontal)
        self.slider.setRange(1, 10)
        self.slider.setValue(2)
        self.slider.setTracking(True)
        self.slider.setTickPosition(QSlider.TicksBothSides)
        self.connect(self.slider, SIGNAL('valueChanged(int)'), self.on_draw)
        
        self.table = QTableWidget()
        
        plot_cb_label = QLabel("Variable to plot:")
        self.plot_combobox = QComboBox()
        self.plot_combobox.addItems(CycleStats.fields)
        self.connect(self.plot_combobox, SIGNAL('currentIndexChanged(int)'), self.on_draw)
        
        self.field_listbox = QListWidget()
        self.field_listbox.setSelectionMode(QAbstractItemView.NoSelection)
        self.field_listbox.addItems(CycleStats.fields)
        for index in xrange(self.field_listbox.count()):
            item = self.field_listbox.item(index)
            item.setCheckState(Qt.CheckState(0))
        self.field_listbox.setMinimumWidth(self.field_listbox.sizeHintForColumn(0))
        self.connect(self.field_listbox, SIGNAL('itemClicked(QListWidgetItem *)'), self.on_draw)
        
        toparea = QSplitter()
        toparea.addWidget(self.field_listbox)
        toparea.addWidget(self.canvas)
        
        
        
        #
        # Layout with box sizers
        # 
        hbox = QHBoxLayout()
        
        for w in [  plot_cb_label, self.plot_combobox, self.grid_cb,
                    slider_label, self.slider]:
            hbox.addWidget(w)
            hbox.setAlignment(w, Qt.AlignVCenter)
        
        plotsection = QVBoxLayout()
        plotsection.addWidget(self.mpl_toolbar)
        plotsection.addWidget(toparea)
        #plotsection.addLayout(hbox)
        plotsectionwidget = QWidget()
        plotsectionwidget.setLayout(plotsection)
        
        vbox = QSplitter(Qt.Orientation.Vertical)
        vbox.addWidget(plotsectionwidget)
        vbox.addWidget(self.table)
        
        bigbox = QVBoxLayout()
        bigbox.addWidget(vbox)
        
        self.main_frame.setLayout(bigbox)
        self.setCentralWidget(self.main_frame)
    
    def create_status_bar(self):
        self.status_text = QLabel("This is a demo")
        self.statusBar().addWidget(self.status_text, 1)
        
    def create_menu(self):        
        self.file_menu = self.menuBar().addMenu("&File")
        
        load_file_action = self.create_action("&Save plot",
            shortcut="Ctrl+S", slot=self.save_plot, 
            tip="Save the plot")
        quit_action = self.create_action("&Quit", slot=self.close, 
            shortcut="Ctrl+Q", tip="Close the application")
        
        self.add_actions(self.file_menu, 
            (load_file_action, None, quit_action))
        
        self.help_menu = self.menuBar().addMenu("&Help")
        about_action = self.create_action("&About", 
            shortcut='F1', slot=self.on_about, 
            tip='About the demo')
        
        self.add_actions(self.help_menu, (about_action,))

    def add_actions(self, target, actions):
        for action in actions:
            if action is None:
                target.addSeparator()
            else:
                target.addAction(action)

    def create_action(  self, text, slot=None, shortcut=None, 
                        icon=None, tip=None, checkable=False, 
                        signal="triggered()"):
        action = QAction(text, self)
        if icon is not None:
            action.setIcon(QIcon(":/%s.png" % icon))
        if shortcut is not None:
            action.setShortcut(shortcut)
        if tip is not None:
            action.setToolTip(tip)
            action.setStatusTip(tip)
        if slot is not None:
            self.connect(action, SIGNAL(signal), slot)
        if checkable:
            action.setCheckable(True)
        return action

def plot_csts(cst_list):
    app = QApplication(sys.argv)
    form = AppForm(cst_list)
    form.show()
    app.exec_()    


def main():
    app = QApplication(sys.argv)
    form = AppForm(None)
    form.show()
    app.exec_()


if __name__ == "__main__":
    main()


__author__ = 'andrew'

from timeutil import *


class ModemLog(dict):
        fields = ('finished', 'message_number', 'datetime',
              'logged_message')

        # This automagically retrieves values from the dictionary when they are referenced as properties.
        # Whether this is awesome or sucks is open to debate.
        def __getattr__(self, item):
            """Maps values to attributes.
            Only called if there *isn't* an attribute with this name
            """
            try:
                return self.__getitem__(item)
            except KeyError:
                raise AttributeError(item)

        def __setattr__(self, item, value):
            """Maps attributes to values.
            Only if we are initialised
            """
            if not self.__dict__.has_key('_CycleStats__initialized'):  # this test allows attributes to be set in the __init__ method
                return dict.__setattr__(self, item, value)
            elif self.__dict__.has_key(item):       # any normal attributes are handled normally
                dict.__setattr__(self, item, value)
            else:
                self.__setitem__(item, value)

        #For backward compatibility with classes that directly access the values dict.
        @property
        def values(self):
            return self

        def __str__(self):
            ''' Default human-readable version of ModemLog.
            Doesn't show all parameters, just the most useful ones.'''
            hrstr = "{dt}\tMessage: {msg}\t".format(
                        dt=self['datetime'], msg=self['logged_message'])

            return hrstr

        @classmethod
        def from_nmea_msg(cls, msg):

            values = dict.fromkeys(ModemLog.fields)
            values['finished'] = int(msg["params"][0])
            values['message_number'] = int(msg["params"][1])
            values['datetime']= convert_to_datetime(msg["params"][2])
            values['logged_message'] = (msg["params"][3:])

            log = cls(values)
            return log



'''
Created on Jan 23, 2012

@author: Eric
'''
import datetime


class CycleStats(dict):
    '''
    A single set of Receive Cycle Statistics
    '''
    
    # Note that we don't override the dict initalizer.
    
    fields = ('mode', 'toa', 'toa_mode', 
              'mfd_peak', 'mfd_pow', 'mfd_ratio', 'mfd_spl', 
              'agn', 'shift_ain', 'shift_ainp', 'shift_mfd', 'shift_p2b',
              'rate_num', 'src', 'dest', 'psk_error', 'packet_type',
              'num_frames', 'bad_frames_num', 
              'snr_rss', 'snr_in', 'snr_out', 'snr_sym', 
              'mse', 'dqf', 'dop', 'noise', 'carrier', 'bandwidth', 
              'version_number')    
    
    # This automagically retrieves values from the dictionary when they are referenced as properties.
    # Whether this is awesome or sucks is open to debate.
    def __getattr__(self, item):
        """Maps values to attributes.
        Only called if there *isn't* an attribute with this name
        """
        try:
            return self.__getitem__(item)
        except KeyError:
            raise AttributeError(item)
        
    def __setattr__(self, item, value):
        """Maps attributes to values.
        Only if we are initialised
        """
        if not self.__dict__.has_key('_CycleStats__initialized'):  # this test allows attributes to be set in the __init__ method
            return dict.__setattr__(self, item, value)
        elif self.__dict__.has_key(item):       # any normal attributes are handled normally
            dict.__setattr__(self, item, value)
        else:
            self.__setitem__(item, value)    

    #For backward compatibility with classes that directly access the values dict.
    @property
    def values(self):
        return self

        
    def __str__(self):
        ''' Default human-readable version of CycleStats. 
        Doesn't show all parameters, just the most common ones.'''
        hrstr = "{ts}\tRate: {rate_num:.0f}\t".format(
                    ts=self['toa'], rate_num=self['rate_num'])
        if self['rate_num'] is 0:
            hrstr = hrstr + "DQF: {dqf:.0f}\tBad Frames: {bad_frames:.0f}".format(dqf=self['dqf'], bad_frames=self['bad_frames_num'])
        if self['rate_num'] in range(1,7):
            hrstr = hrstr + "PSK Error: {psk_error:.0f}\tBad Frames: {bad_frames:.0f}\tInput SNR: {snr_in:.1f}\tMSE: {mse:.1f}".format(
                        snr_in=self['snr_in'], mse=self['mse'], bad_frames=self['bad_frames_num'], psk_error=self['psk_error'])
                   
        return hrstr
    
    
    @classmethod
    def from_nmea_msg(cls, msg, log_datetime=None, drop_packet_timeout=True):
        
        values = dict.fromkeys(CycleStats.fields)

        
        version_number = int(msg['params'][0])
        if( version_number >=6):
            values['version_number'] = version_number
            values['mode'] = int(msg['params'][1])

            
            # Parse the TOA field into a fractional datetime object.
            whole, fract = str(msg['params'][2]).split('.')
            toa = datetime.datetime.strptime(whole, '%Y%m%d%H%M%S')
            values['toa'] = toa.replace(microsecond = int(fract))
            
            # If we have a PACKET_TIMEOUT, just return with "None" in most fields
            if values['mode'] == 2:
                if drop_packet_timeout:
                    return None
                else:
                    return cls(values)            

            values['toa_mode'] = int(msg['params'][3])
            values['mfd_peak'] = int(msg['params'][4])
            values['mfd_pow'] = int(msg['params'][5])
            values['mfd_ratio'] = int(msg['params'][6])
            values['mfd_spl'] = int(msg['params'][7])
            values['agn'] = int(msg['params'][8])
            values['shift_ain'] = int(msg['params'][9])
            values['shift_ainp'] = int(msg['params'][10])
            values['shift_mfd'] = int(msg['params'][11])
            values['shift_p2b'] = int(msg['params'][12])               
            values['rate_num'] = int(msg['params'][13])
            values['src'] = int(msg['params'][14])
            values['dest'] = int(msg['params'][15])
            values['psk_error'] = int(msg['params'][16])
            values['packet_type'] = int(msg['params'][17])
            values['num_frames'] = int(msg['params'][18])
            values['bad_frames_num'] = int(msg['params'][19])
            values['snr_rss'] = int(msg['params'][20])
            values['snr_in'] = float(msg['params'][21])
            values['snr_out'] = float(msg['params'][22])
            values['snr_sym'] = float(msg['params'][23])
            values['mse'] = float(msg['params'][24])
            values['dqf'] = int(msg['params'][25])
            values['dop'] = float(msg['params'][26])
            values['noise'] = int(msg['params'][27])
            values['carrier'] = int(msg['params'][28])
            values['bandwidth'] = int(msg['params'][29])


        else:
            version_number = 0
            values['mode'] = int(msg['params'][0])
            
            
            if log_datetime is not None:
                values['toa'] = log_datetime
            else:
                # Use today's date, since this version of the CST message doesn't include a date.
                # Also, don't bother parsing the fractional seconds for the uM1.
                toastr = str(msg['params'][1])
                values['toa'] = datetime.datetime.combine(
                    datetime.date.today(), datetime.time(int(toastr[0:2]), 
                                                         int(toastr[2:4]), 
                                                         int(toastr[4:6])))
            
            # If we have a PACKET_TIMEOUT, just return with "None" in most fields
            if values['mode'] == 2:
                if drop_packet_timeout:
                    return None
                else:
                    return cls(values)
            
            values['toa_mode'] = -100
            values['mfd_peak'] = int(msg['params'][3])
            values['mfd_pow'] = int(msg['params'][4])
            values['mfd_ratio'] = int(msg['params'][5])
            values['mfd_spl'] = int(msg['params'][6])
            values['agn'] = int(msg['params'][7])
            values['shift_ain'] = int(msg['params'][8])
            values['shift_ainp'] = int(msg['params'][9])
            values['shift_mfd'] = int(msg['params'][10])
            values['shift_p2b'] = int(msg['params'][11])               
            values['rate_num'] = int(msg['params'][12])
            values['src'] = int(msg['params'][13])
            values['dest'] = int(msg['params'][14])
            values['psk_error'] = int(msg['params'][15])
            values['packet_type'] = int(msg['params'][16])
            values['num_frames'] = int(msg['params'][17])
            values['bad_frames_num'] = int(msg['params'][18])
            values['snr_rss'] = int(msg['params'][19])
            values['snr_in'] = float(msg['params'][20])
            values['snr_out'] = float(msg['params'][21])
            values['snr_sym'] = float(msg['params'][22])
            values['mse'] = float(msg['params'][23])
            values['dqf'] = int(msg['params'][24])
            values['dop'] = float(msg['params'][25])
            if len(msg['params']) >= 27:
                values['noise'] = int(msg['params'][26])
                version_number = 3
            else:
                values['noise'] = -100
            
            if len(msg['params']) >= 28:
                values['carrier'] = int(msg['params'][27])
                values['bandwidth'] = int(msg['params'][28])
                version_number = 4
            else:
                values['carrier'] = 0
                values['bandwidth'] = 0                  
            
            values['version_number'] = version_number
        
                   
        # Make a CycleStats
        cst = cls(values)
    
        return cst

class CycleStatsList(list):
    # We want to do list-ish things, with some extra sauce.
    
    def to_dict_of_lists(self):
        dol = {}        
        for field in CycleStats.fields:
            dol[field] = [cst[field] for cst in self]
        
        return dol

    # This automagically retrieves lists for each parameter when they are referenced as properties.
    # Whether this is awesome or sucks is open to debate.
    def __getattr__(self, item):
        """Maps values to attributes.
        Only called if there *isn't* an attribute with this name
        """
        if item in CycleStats.fields:
            return [cst[item] for cst in self]
        else:
            raise AttributeError(item)
    
            
class TransmitStats(dict):
    '''
    A single set of Receive Cycle Statistics
    '''

    # Note that we don't override the dict initalizer.

    fields = ('time','toa_mode','mode', 'probe_length',
              'bandwidth', 'carrier', 'rate_num', 'src', 'dest',
              'ack', 'num_frames_expected', 'num_frames_sent', 'packet_type',
              'nbytes', 'version_number')

    # This automagically retrieves values from the dictionary when they are referenced as properties.
    # Whether this is awesome or sucks is open to debate.
    def __getattr__(self, item):
        """Maps values to attributes.
        Only called if there *isn't* an attribute with this name
        """
        try:
            return self.__getitem__(item)
        except KeyError:
            raise AttributeError(item)

    def __setattr__(self, item, value):
        """Maps attributes to values.
        Only if we are initialised
        """
        if not self.__dict__.has_key('_TransmitStats__initialized'):  # this test allows attributes to be set in the __init__ method
            return dict.__setattr__(self, item, value)
        elif self.__dict__.has_key(item):       # any normal attributes are handled normally
            dict.__setattr__(self, item, value)
        else:
            self.__setitem__(item, value)

    #For backward compatibility with classes that directly access the values dict.
    @property
    def values(self):
        return self


    def __str__(self):
        ''' Default human-readable version of CycleStats.
        Doesn't show all parameters, just the most common ones.'''
        hrstr = "{ts}\tRate: {rate_num:.0f}\tFrames: {NFSent}/{NFExp}\t# Bytes: {nbytes}".format(
                    ts=self['time'], rate_num=self['rate_num'],
                    NFSent=self['num_frames_sent'],NFExp=self['num_frames_expected'],
                    nbytes=self['nbytes'])
        return hrstr


    @classmethod
    def from_nmea_msg(cls, msg, log_datetime=None, drop_data_timeout=True):

        values = dict.fromkeys(CycleStats.fields)


        version_number = int(msg['params'][0])
        if( version_number ==6):
            values['version_number'] = version_number

            date = datetime.datetime.strptime(str(msg['params'][1]), "%Y%m%d")

            # Parse the time field into a fractional datetime object.
            whole, fract = str(msg['params'][2]).split('.')
            time = datetime.datetime.strptime(whole, '%H%M%S')
            values['time'] = time.replace(year=date.year,month=date.month,day=date.day,microsecond = int(fract))

            values['toa_mode'] = int(msg['params'][3])
            values['mode'] = int(msg['params'][4])
            # If we have a PACKET_TIMEOUT, just return with "None" in most fields
            if values['mode'] == 4:
                if drop_data_timeout:
                    return None
                else:
                    return cls(values)




            values['probe_length'] = int(msg['params'][5])
            values['bandwidth'] = int(msg['params'][6])
            values['carrier'] = int(msg['params'][7])
            values['rate_num'] = int(msg['params'][8])
            values['src'] = int(msg['params'][9])
            values['dest'] = int(msg['params'][10])
            values['ack'] = int(msg['params'][11])
            values['num_frames_expected'] = int(msg['params'][12])
            values['num_frames_sent'] = int(msg['params'][13])
            values['packet_type'] = int(msg['params'][14])
            values['nbytes'] = int(msg['params'][15])

        else:
            version_number = 0
            date = datetime.datetime.strptime(str(msg['params'][0]), "%Y%m%d")

            # Parse the time field into a fractional datetime object.
            whole, fract = str(msg['params'][1]).split('.')
            time = datetime.datetime.strptime(whole, '%H%M%S')
            values['time'] = time.replace(year=date.year,month=date.month,day=date.day,microsecond = int(fract))

            values['toa_mode'] = int(msg['params'][2])
            values['mode'] = int(msg['params'][3])
            # If we have a PACKET_TIMEOUT, just return with "None" in most fields
            if values['mode'] == 4:
                if drop_data_timeout:
                    return None
                else:
                    return cls(values)

            values['probe_length'] = int(msg['params'][4])
            values['bandwidth'] = int(msg['params'][5])
            values['carrier'] = int(msg['params'][6])
            values['rate_num'] = int(msg['params'][7])
            values['src'] = int(msg['params'][8])
            values['dest'] = int(msg['params'][9])
            values['ack'] = int(msg['params'][10])
            values['num_frames_expected'] = int(msg['params'][11])
            values['num_frames_sent'] = int(msg['params'][12])
            values['packet_type'] = int(msg['params'][13])
            values['nbytes'] = int(msg['params'][14])

            values['version_number'] = version_number


        # Make a CycleStats
        xst = cls(values)

        return xst

class TransmitStatsList(list):
    # We want to do list-ish things, with some extra sauce.

    def to_dict_of_lists(self):
        dol = {}
        for field in TransmitStats.fields:
            dol[field] = [xst[field] for xst in self]

        return dol

    # This automagically retrieves lists for each parameter when they are referenced as properties.
    # Whether this is awesome or sucks is open to debate.
    def __getattr__(self, item):
        """Maps values to attributes.
        Only called if there *isn't* an attribute with this name
        """
        if item in TransmitStats.fields:
            return [xst[item] for xst in self]
        else:
            raise AttributeError(item)


__author__ = 'andrew'

import abc

class NmeaListener(object):
    ''' Abstract class for connecting for processing Nmea Message via some connection (such as serial, TCP, etc).
    '''

    metaclass__ = abc.ABCMeta

    @abc.abstractmethod
    def _process_outgoing_nmea(self):
        pass

    @abc.abstractmethod
    def _process_incoming_nmea(self, msg):
        pass


    @abc.abstractmethod
    def _message_waiting(self):
        pass



'''
Created on Jul 13, 2012

@author: Eric
'''

from threading import Thread
from time import sleep

from serial import Serial
from acomms.modem_connections.serial_connection import SerialConnection
from Queue import Empty


class IridiumConnection(SerialConnection):
    '''
    classdocs
    '''


    def __init__(self, modem, port, baudrate, number, timeout=0.1):
        '''
        Constructor
        '''
        self._incoming_line_buffer = ""

        self.connection_type = "direct_iridium"

        self.modem = modem
        self.port = port
        self.baudrate = baudrate
        self.timeout = timeout

        self._serialport = Serial(port, baudrate, timeout=self.timeout)

        self._thread = Thread(target=self._listen)
        self._thread.setDaemon(True)
        self._thread.start()

        self.state = 'DISCONNECTED'
        self.modem = modem
        self.number = str(number)
        self.counter = 0

    @property
    def is_connected(self):
        return self._serialport.getCD()

    @property
    def can_change_baudrate(self):
        return True

    def change_baudrate(self,baudrate):
        return self.baudrate


    def _listen(self):
        while True:
            if self._serialport.isOpen():
                msg = self.readline()
                if not self._serialport.getCD():
                    # Not connected via Iridium
                    # Processing I/O with Iridium dialer.
                    self.process_io(msg)
                else:
                    # We are connected, so pass through to NMEA
                    self.modem._process_incoming_nmea(msg)
                    self.modem._process_outgoing_nmea()
            else:  # not connected
                sleep(0.5) # Wait half a second, try again.

    def process_io(self, msg):
        # This is called by the primary serial processing loop.
        # It will be called whenever we have a line of data, or periodically (based on a timeout)
        if msg is None:
            msg = ""

        if self.state == "DIALING":
            if "CONNECT" in msg:
                self.state = "CONNECTED"
            elif "NO CARRIER" in msg:
                self.state = "DISCONNECTED"
            elif "NO ANSWER" in msg:
                self.state = "DISCONNECTED"
            elif "BUSY" in msg:
                self.state = "DISCONNECTED"
            elif "ERROR" in msg:
                self.state = "DISCONNECTED"
            if self.counter > 600: #Counts are about 0.1s
                self.state = "DISCONNECTED"
                self.modem._daemon_log.info("$IRIDIUM,{0},Dialing Attempt Timed Out.".format(self.modem.name))
            self.counter += 1
        elif self.state == 'DISCONNECTED':
            self.counter = 0
            self.do_dial()
        elif self.state == "CONNECTED":
            # In theory, we shouldn't be here, because if we are connected, traffic is passed through to the umodem module.
            # So, give us a 1 message margin of error (basically, ignore this input) and try dialing on the next timeout/message.
            self.state = "DISCONNECTED"

        if msg is not "":
            self.modem._daemon_log.info("$IRIDIUM,{0},{1}".format(self.modem.name, msg.strip()))
        self.modem._daemon_log.debug("$IRIDIUM,{0},Current State:{1}".format(self.modem.name, self.state))

    def do_dial(self):
        # Toggle DTR
        self.modem._daemon_log.info("$IRIDIUM,{0},Dialing {1}".format(self.modem.name, self.number))
        sleep(2)
        self._serialport.setDTR(False)
        sleep(0.1)
        self._serialport.setDTR(True)
        sleep(0.1)
        self._serialport.write("AT+CREG?\r\n")
        sleep(1)
        self._serialport.write("AT+CEER\r\n")
        sleep(1)
        self._serialport.write("AT+CSQ?\r\n")
        sleep(5)
        self._serialport.write("ATD{0}\r\n".format(self.number))
        self.state = "DIALING"

    def close(self):
        self._serialport.setDTR(False)
        sleep(0.2)
        self._serialport.close()

    def wait_for_connect(self):
        while self.state != "CONNECTED":
            sleep(1)


from __future__ import print_function
from acomms import CycleStats, CycleStatsList, Message
import dateutil.parser
import datetime
from collections import namedtuple
import scipy.io
import numpy
import glob
import pickle
import argparse
import csv
import operator
import plotter

CstParseResult = namedtuple('CstParseResult', 'cst_list line_count error_count')

def _bareprint(text):
    print(text, sep='', end='')
   
def get_csts_from_log_file(log_filename, console_progress=False, drop_packet_timeout=True):
    """ Get a CycleStatsList containing the data from all the CACST messages in 
    the specified log file.
    
    This function attempt to deal with multiple log file formats and do the
    "right thing".  If the log file contains pre-version-6 CST messages that
    don't include a full date, it will try to get the date/time from the log
    file timestamp instead.
    
    If console_progress is set, this function will print progress information to
    the console.
    
    Returns a named tuple with the CycleStatsList, the number of lines
    processed, and the number of lines where a parsing error occurred.
    """
    
    line_count = 0
    error_count = 0
    cst_list = CycleStatsList()
    
    if console_progress:
        _bareprint("Parsing {}...".format(log_filename))
                   
    with open(log_filename, 'r') as logfile:
        for line in logfile:
            line_count += 1
            
            if console_progress:
                if (line_count % 1000 == 0):
                    _bareprint(".")
                    
            dollar_pos = line.find("$CACST")
            # str.find returns -1 if the substring was not found
            if dollar_pos >= 0:
                try:
                    # look for a date in the first part of the string
                    # We start removing (hopefully garbage) characters
                    for remove_idx in range(dollar_pos):
                        try:
                            line_timestamp = dateutil.parser.parse(line[:(dollar_pos-remove_idx)], fuzzy=True)
                            break
                        except ValueError:
                            continue
                    msg = Message(line[dollar_pos:])
                    cst = CycleStats.from_nmea_msg(msg, log_datetime=line_timestamp, drop_packet_timeout=drop_packet_timeout)
                    # For now, drop packet timeouts
                    if cst.mode != 2:
                        cst_list.append(cst)
                except:
                    # We should do something, but we won't.
                    error_count += 1
    
    if console_progress:
        print("Done.\n  Processed {} lines, parsed {} CSTs, encountered {} errors.".format(line_count, len(cst_list), error_count))
    
    return CstParseResult(cst_list, line_count, error_count)

def get_csts_from_log_files(log_filename_list, console_progress=False, drop_packet_timeout=True):
    """ Get a CycleStatsList containing the data from all the CACST messages in 
    the specified log files.
    
    This function attempt to deal with multiple log file formats and do the
    "right thing".  If the log file contains pre-version-6 CST messages that
    don't include a full date, it will try to get the date/time from the log
    file timestamp instead.
    
    If console_progress is set, this function will print progress information to
    the console.
    
    Returns a named tuple with the CycleStatsList, the total number of lines
    processed, and the total number of lines where a parsing error occurred.
    """
    
    total_line_count = 0
    total_error_count = 0
    all_csts_list = CycleStatsList()
    
    for filename in log_filename_list:
        cst_list, line_count, error_count = get_csts_from_log_file(filename, console_progress)
        all_csts_list.extend(cst_list)
        total_error_count += error_count
        total_line_count += line_count
    
    if console_progress:    
        print("Processed {} files, {} total lines, {} total CSTs, {} parsing errors.".format(
            len(log_filename_list), total_line_count, len(all_csts_list), total_error_count))
      
    return CstParseResult(all_csts_list, total_line_count, total_error_count)


def save_matlab_from_csts(mat_file_name, cst_list, variable_name="csts"):
    cst_dict_of_lists = cst_list.to_dict_of_lists()
    # We can't save a normal datetime to a MAT file, so we have to convert
    # it to a numpy datetime64.
    cst_dict_of_lists['toa'] = [numpy.datetime64(toa) for toa in cst_dict_of_lists['toa']]
    
    scipy.io.savemat(mat_file_name, mdict={variable_name: cst_dict_of_lists}, oned_as='row')
    
def save_pickle_from_csts(pickle_file_name, cst_list, pickle_protocol=2):
    with open(pickle_file_name, 'wb') as picklefile:
        pickle.dump(cst_list, picklefile, protocol=pickle_protocol)
        
def save_csv_from_csts(csv_file_name, cst_list, write_header=True):
    # "If csvfile is a file object, it must be opened with the b flag on platforms where that makes a difference."
    with open(csv_file_name, 'wb') as csvfile:
        dw = csv.DictWriter(csvfile, CycleStats.fields, extrasaction='ignore')
        if write_header:
            dw.writeheader()
        dw.writerows(cst_list)
        
def load_csts_from_pickle(pickle_file_name):
    with open(pickle_file_name, 'rb') as picklefile:
        cst_list = pickle.load(picklefile)
        return cst_list

def print_csts_to_console(cst_list):
    # This is probably a bad idea.
    for cst in cst_list:
        print(cst)
    

if __name__ == '__main__':
    ap = argparse.ArgumentParser(description='Magically transform CST messages into something useful.')
    ap.add_argument("-m", "--matfile", help="Save Matlab .MAT file with specified name")
    ap.add_argument("-k", "--pickle", help="Save Python pickle file with specified name")
    ap.add_argument("-c", "--csv", help="Save csv file with specified name")
    ap.add_argument("-p", "--console", action='store_true', help="Print human-readable CST values to console")
    ap.add_argument("-g", "--gui", action='store_true', help="Plot CSTs in interactive GUI (experimental)")
    ap.add_argument("-l", "--load", action='store_true', help="Load CSTs from pickle file instead of parsing log files")
    ap.add_argument("--silent", action='store_true', help="Don't print progress messages")
    ap.add_argument("--sort", default="oldfirst", choices=["none", "oldfirst", "newfirst"], help="Sort the CSTs by time in the specified direction prior to generating output. Default is 'oldfirst'.")
    ap.add_argument("log_filenames", nargs='+', help="File name(s) of log files to process.  Wildcards are OK.")
    
    args = ap.parse_args()
    
    filename_list = []
    for filename in args.log_filenames:
        filename_list.extend(glob.glob(filename))
        
    show_progress = not args.silent
    
    # First, try to parse the files we were given.
    if args.load:
        if show_progress:
            _bareprint("Loading CSTs from {}...".format(filename_list[0]))
        cst_list = load_csts_from_pickle(filename_list[0])
        if show_progress:
            print("Done.")
    else:
        cst_list = get_csts_from_log_files(filename_list, console_progress=show_progress)[0]
    
    if args.sort is not "none":
        if args.sort is "oldfirst":
            reverse = False
        else:
            reverse = True
        cst_list.sort(key=operator.itemgetter('toa'), reverse=reverse)
       
    # Now, choose what to do.
    if args.console:
        print_csts_to_console(cst_list)
    
    if args.matfile:
        if show_progress:
            _bareprint("Writing Matlab MAT file to {}...".format(args.matfile))
        save_matlab_from_csts(args.matfile, cst_list)
        if show_progress:
            print("Done.")
    
    if args.pickle:
        if show_progress:
            _bareprint("Writing pickle file to {}...".format(args.pickle))
        save_pickle_from_csts(args.pickle, cst_list)
        if show_progress:
            print("Done.")
        
    if args.csv:
        if show_progress:
            _bareprint("Writing CSV file to {}...".format(args.csv))
        save_csv_from_csts(args.csv, cst_list)
        if show_progress:
            print("Done.")
    
    if args.gui:
        print("Starting Magic CST GUI (this may take a moment)...")
        plotter.plot_csts(cst_list)

    

        
        



__author__ = 'Eric'

from modem_connection import ModemConnection
from iridium_connection import IridiumConnection
from serial_connection import SerialConnection
from sbd_connection import SBDEmailConnection
from tcp_connection import TcpConnection



__author__ = 'andrew'

#This is an implementation of the Xmodem Protocol modified for the Microcmodem IO methods.
from messageparams import Rates, FDPRates, data_from_hexstring, hexstring_from_data
from unifiedlog import UnifiedLog
from acomms import Micromodem
from Queue import Queue,Empty,Full
import crcmod
import time
import os

class acomms_xymodem(object):
    '''
    XMODEM Like Protocol handler, expects an object to read from and an object to
    write to.
    :type mode: string
    :param pad: Padding character to make the packets match the packet size
    :type pad: char
    '''

    def __init__(self, micromodem, dest,rate = 1, timeout=60,pad='\x1a',ymodem_enabled = True,unified_log=None,log_path=None, ):
        assert isinstance(micromodem, Micromodem), "micromodem object isn't a Micromodem: %r" & micromodem
        assert rate in range(0,7), "Invalid Rate: %d" & rate
        assert dest != micromodem.id, "Can't send file to self."
        self.pad = pad

        self.rate = Rates[rate]
        self.micromodem = micromodem
        self.micromodem.set_config('RXP',1)
        self.micromodem.set_config('MOD',1)
        #use CRC-8/CDMA2000 instead of CRC-16 CCITT Unreflected.
        self.calc_crc = crcmod.mkCrcFun(0x19b, rev=False, initCrc=0xff, xorOut=0x00)
        if unified_log is None:
            unified_log = UnifiedLog(log_path=log_path)
        self.log = unified_log.getLogger("xmodem.{0}".format(micromodem.name))
        self.target_id = dest
        self.timeout = timeout
        self.ymodem_mode = True
        self.use_minipackets = False
        self.fsk_mode = self.rate.number == 0
        self.micromodem.ack_listeners.append(self.ack_recv)
        self.ack_list = Queue()

    SOH = '0001'
    STX = '0002'
    EOT = '0004'
    ACK = '0006'
    DLE = '0010'
    NAK = '0015'
    CAN = '0018'
    CRC = '0043'

    CRC_BIT_ID = 0xF000
    SEQ_NUM_BIT_ID = 0x0F00
    MAX_SEQ_NUM = 256

    def ack_recv(self,ack,msg):
        self.acv_list.push(ack)


    def _receive_across_link(self,ack,timeout=None,delay=3,force_packet = False, validate_packet=False):
        if self.use_minipackets and not force_packet:
            data = self.micromodem.wait_for_minipacket(timeout=timeout)
            if ack:
                print("Acking Minipacket Data: {}".format(data))
                self.micromodem.send_minipacket(dest_id=self.target_id,databytes=self.ACK)
                self.micromodem.wait_for_xst(timeout)
                time.sleep(delay)
        else:
            data = self.micromodem.wait_for_data_packet(fsk=self.fsk_mode,timeout=timeout)
            if ack:
                print("Waiting for Ack to Finish Transmitting")
                self.micromodem.wait_for_xst(timeout=timeout)
        return data

    def _send_across_link(self,data,ack,timeout = None,delay = 3,force_packet = False, validate_packet=False):
        if self.use_minipackets and not force_packet:
            self.micromodem.send_minipacket(dest_id=self.target_id,databytes=data)
            self.micromodem.wait_for_xst(timeout)
            time.sleep(delay)
            if ack:
                print ("Waiting for Minpacket ACK")
                char = self.micromodem.wait_for_minipacket(timeout=None)
                if char != self.ACK:
                    print "Nonack character received: {}".format(char)
                    return False
            return True
        else:
            self.micromodem.send_packet_data(dest=self.target_id,rate_num=self.rate.number,databytes=data,ack=ack)
            xst = self.micromodem.wait_for_xst(timeout=None)
            if validate_packet:
                print("Validating Packet Transmission")
                if xst['num_frames_sent'] != xst['num_frames_expected']:
                    print("num_frames_sent != num_frames_expected: {} != {}".format(xst['num_frames_sent'] , xst['num_frames_expected']))
                    return False
            if ack:
                print("Waiting for {} Acks".format(xst['num_frames_sent']))
                self.micromodem.ack_listeners.append(self.ack_recv)
                self.micromodem.wait_for_cst(timeout=timeout)
                #for i in range(1,xst['num_frames_sent']+1):
                #    ack_reply = self.micromodem.wait_for_nmea_type('CAACK',timeout=10)
                #    if ack_reply is None:
                #        print "No ACK received for frame {}.".format(i)
                #        return False
                    #if int(ack_reply['params'][2]) != int(i):
                    #    print "Ack Received not for current frame ({} != {}).".format(ack_reply['params'][2],i)
                    #    return False
                    #print("Received Ack: {}".format(ack_reply))
                self.micromodem.ack_listeners.remove(self.ack_recv)
                frames = range(1,xst['num_frames_sent']+1)
                while True:
                    try:
                        ack_recv = self.ack_list.pop()
                        if ack_recv.frame_num in frames:
                            frames.remove(ack_recv.frame_num)
                    except Empty:
                        break

                if not frames:
                        print "No ACK received for frames {}.".format(frames)
                        return False
            return True

    def calculate_packet_delay(self,timeout=30):
        #determine distance between nodes for 1-way travel time delay estimates.
        self.micromodem.send_ping(dest_id=self.target_id)
        delay = int(self.micromodem.wait_for_ping_reply(dest_id=self.target_id,timeout=timeout))  + 3 #Each packet takes roughly 3 seconds to transmit


    def abort(self, count=2, timeout=60):
        '''
        Send an abort sequence using CAN bytes.
        '''
        for counter in xrange(0, count):
            print ("Sending CAN")
            self._send_across_link(self.CAN,False,timeout=timeout)


    def send(self, file_io, retry=16, timeout=15, delay = 1, callback=None):
        '''
        Send a stream via the XMODEM Like protocol.

            >>> stream = file('/etc/issue', 'rb')
            >>> print modem.send(stream)
            True

        Returns ``True`` upon succesful transmission or ``False`` in case of
        failure.

        :param stream: The stream object to send data from.
        :type stream: stream (file, etc.)
        :param retry: The maximum number of times to try to resend a failed
                      packet before failing.
        :type retry: int
        :param timeout: The number of seconds to wait for a response before
                        timing out.
        :type timeout: int
        :param quiet: If 0, it prints info to stderr.  If 1, it does not print any info.
        :type quiet: int
        :param callback: Reference to a callback function that has the
                         following signature.  This is useful for
                         getting status updates while a xmodem
                         transfer is underway.
                         Expected callback signature:
                         def callback(total_packets, success_count, error_count)
        :type callback: callable
        '''
        assert delay != None, "Invalid Delay."
        if self.ymodem_mode:
            assert os.path.isfile(file_io), "Ymodem Mode Enabled and File not passed in."
        else:
            assert hasattr(file_io,"read"), "Stream not readable. "
        assert timeout > delay, "Timeout is less than delay."
        if callback != None:
            assert callable(callback), "Non-nulll callback isn't callable."


        # initialize protocol
        try:
            packet_size = self.rate.getpacketsize()
        except AttributeError:
            raise ValueError("An invalid mode was supplied")


        if self.ymodem_mode:
            stream = open(file_io,'r+')
        else:
            stream = file_io

        error_count = 0
        crc_mode = 0
        cancel = 0

        #Wait for minipacket containing control character

        while True:
            print("Waiting On Connection")
            minipacket_data = self._receive_across_link(ack=False)

            if minipacket_data:
                if minipacket_data == self.NAK:
                    print("CRC Mode Off")
                    crc_mode = 0
                    break
                elif minipacket_data == self.CRC:
                    print("CRC Mode On, Acking")
                    crc_mode = 1
                    self._send_across_link(data=self.ACK,ack=False,timeout=None,delay=delay)
                    break
                elif minipacket_data == self.CAN:
                    print('received CAN')
                    if cancel:
                        return False
                    else:
                        cancel = 1
                else:
                    print('send ERROR expected NAK/CRC, got %s' % (minipacket_data))

            error_count += 1
            if error_count >= retry:
                self.abort(timeout=timeout)
                return False

        #Send Packet Header.

        # send data
        error_count = 0
        success_count = 0
        total_packets = 0
        sequence = 0
        while True:
            #Send Packet Header if in that mode.
            if sequence == 0 and self.ymodem_mode:
                data = os.path.basename(file_io).lower() + " " + \
                       str(os.path.getsize(file_io))
            #Otherwise just send data
            else:
                data = stream.read(packet_size)
            if not data:
                print('No more data. End of Stream')
                # end of stream
                break
            total_packets += 1
            #data = data.ljust(packet_size, self.pad)
            if crc_mode:
                crc = self.calc_crc(data)
            # emit packet
            while True:
                print("Starting Transmission")
                self._send_across_link(data=self.STX,ack=False,timeout=None,delay=delay)

                #Send our sequence number
                sequence_num = sequence + self.SEQ_NUM_BIT_ID #Add Sequence Bit ID to the front.
                print("Sending Sequence Number: {} ({})".format(sequence_num, "{:04X}".format(sequence_num)))
                ok = self._send_across_link(data="{:04X}".format(sequence_num),ack=True,timeout=None,delay=delay)
                #Since Modem is broken on larger than 8 bit mini-packets, send a real packet.
                if not ok:
                    print('Sequence number not acked, Aborting')
                    self.abort(timeout=timeout)
                    return False

                print ("Sending Data.")
                ok = self._send_across_link(data=data, ack=True, force_packet=True, validate_packet=True)
                if not ok:
                    error_count += 1
                    print('Not all frames transfered.')
                    if error_count >= retry:
                        # excessive amounts of retransmissions requested,
                        # abort transfer
                        self.abort(timeout=timeout)
                        return False
                    continue

                if crc_mode:
                    print("Sending CRC: {} ({:04X})".format(crc, crc + self.CRC_BIT_ID))
                    self._send_across_link(data="{:04X}".format(crc + self.CRC_BIT_ID),ack=True,timeout=None,delay=delay)

                print("Waiting For Ack.")
                char = self._receive_across_link(ack=False,timeout=None)
                if char == self.ACK:
                    print("Packet Acked.")
                    success_count += 1
                    if callback != None:
                        callback(total_packets, success_count, error_count)
                    # keep track of sequence
                    sequence = (sequence + 1) % self.MAX_SEQ_NUM
                    break
                if char == self.NAK:
                    print("Packet Nacked.")
                    error_count += 1
                    if callback != None:
                        callback(total_packets, success_count, error_count)
                    if error_count >= retry:
                        # excessive amounts of retransmissions requested,
                        # abort transfer
                        self.abort(timeout=timeout)
                        print('excessive NAKs, transfer aborted')
                        return False

                    # return to loop and resend
                    continue
                if char is None:
                    error_count +=1

                # protocol error
                self.abort(timeout=timeout)
                print('protocol error')
                return False

        while True:
            print("Sending EOT")
            # end of transmission
            self._send_across_link(data=self.EOT,ack=False,timeout=None,delay=delay)
            ok = self._receive_across_link(ack=False,timeout=None)
            if ok == self.ACK:
                print("File Transfer Success.")
                break
            if ok == self.NAK:
                print("File Transfer Failed")
                return False
            else:
                error_count += 1
                if error_count >= retry:
                    self.abort(timeout=timeout)
                    self.log.warning('EOT was not ACKd, transfer aborted')
                    return False

        return True

    def recv(self, file_io, crc_mode=1, retry=16, timeout=15, delay=1):
        assert delay != None, "Invalid Delay."
        if self.ymodem_mode:
            assert os.path.isdir(file_io), "Ymodem Mode Enabled and Directory for saving not passed in."
        else:
            assert hasattr(file_io,"write"), "Stream not writeable. "
        assert timeout > delay, "Timeout is less than delay."

        if not self.ymodem_mode:
            stream = file_io

        # initiate protocol
        error_count = 0
        char = 0
        cancel = 0
        size = 0
        while True:
            # first try CRC mode, if this fails,
            # fall back to checksum mode
            if error_count >= retry:
                self.abort(timeout=timeout)
                return None
            elif crc_mode and error_count < (retry / 2):
                print("Sending CRC Mode")
                ok = self._send_across_link(data=self.CRC,ack=True,timeout=None,delay=delay)
                if not ok:
                        print("CRC Mode Not Acked. {} Error Count:{}".format(char,error_count))
                        time.sleep(delay)
                        error_count += 1
                        continue
                else:
                        print("CRC Mode Acked.")
            else:
                crc_mode = 0
                print("CRC Mode Off. Sending NACK")
                self._send_across_link(data=self.NAK,ack=False,timeout=None,delay=delay)


            print("Waiting For Start of Transmission.")
            char = self._receive_across_link(ack=False,timeout=None)

            if char == self.STX:
                break
            elif char == self.CAN:
                print("CAN Received")
                if cancel:
                    print("Canceling Transaction")
                    return None
                else:
                    cancel = 1
            else:
                print("WTF??")
                error_count += 1

        # read data
        error_count = 0
        income_size = 0
        if self.ymodem_mode:
            sequence = 0
        else:
            sequence = 1
        cancel = 0
        filename = ""
        while True:
            while True:
                if char == self.STX:
                    print("STX received at beginning of packet.")
                    break
                elif char == self.EOT:
                    # We received an EOT, so send an ACK and return the received
                    # data length
                    print("End of Transmission. Confirming File Transmission.")
                    if not self.ymodem_mode:
                        stream.close()
                    if income_size != size and size != 0:
                        print "Invalid Size: ({} != {}). Nacking".format(income_size,size)
                        self._send_across_link(data=self.NAK,ack=False,timeout=None,delay=delay)
                        os.remove(file_io + '/'+ str(filename))
                        return 0
                    self._send_across_link(data=self.ACK,ack=False,timeout=None,delay=delay)
                    return income_size
                elif char == self.CAN:
                    # cancel at two consecutive cancels
                    print("CAN Received")
                    if cancel:
                        print("Canceling Transaction")
                        return None
                    else:
                        cancel = 1
                else:
                    print 'recv ERROR expected SOH/EOT, got {}'.format(char)
                    error_count += 1
                    if error_count >= retry:
                        self.abort()
                        return None
            # read sequence
            error_count = 0
            cancel = 0
            print("Waiting For Sequence Number.")
            seq = self._receive_across_link(ack=True,timeout=None)


            print("Waiting For Data.")
            data = self._receive_across_link(ack=True,timeout=None,force_packet=True)

            if crc_mode:
                    print("Waiting For CRC.")
                    crc_recv= self._receive_across_link(ack=False,timeout=None)

            real_seq = int("0x{}".format(seq),16)  - self.SEQ_NUM_BIT_ID
            print "Calculated Sequence Num: {}".format(real_seq)
            data = data.rstrip(self.pad)

            #Process Header
            if real_seq == 0 and self.ymodem_mode:
                (filename,size) = data.split()
                print("Saving {}/{} of size: {}".format(file_io,filename,size))
                stream = open(file_io + '/'+ str(filename), 'w+')
                valid = 1
            #Otherwise process data.
            elif real_seq == sequence:
                # sequence is ok, read packet
                # packet_size + checksum
                if crc_mode:
                    csum = int("0x{}".format(crc_recv),16) - self.CRC_BIT_ID
                    calc_csum = self.calc_crc(str(data))
                    print('CRC (%04x <> %04x)' % (csum, calc_csum))
                    valid = csum == calc_csum
                else:
                    if data != None:
                        valid = 1
                    else:
                        valid = 0
            # valid data, append chunk
            if valid:
                if real_seq > 0 :
                    income_size += len(data)
                    stream.write(data)
                    stream.flush()
                print("Acking Data.")
                ok = self._send_across_link(data=self.ACK,ack=False,timeout=None,delay=delay)
                sequence = (sequence + 1) % self.MAX_SEQ_NUM
                char = self._receive_across_link(ack=False,timeout=None,delay=delay)
                continue
            else:
                print('expecting sequence %d, got %d' % (sequence, real_seq))

            # something went wrong, request retransmission
            print("Nacking Data.")
            self._send_across_link(data=self.NAK,ack=False,timeout=None,delay=delay)
            char = self._receive_across_link(ack=False,timeout=None,delay=delay)


class acomms_xmodem(acomms_xymodem):
    def calculate_packet_delay(self, timeout = 30):
        super(acomms_xmodem, self).calculate_packet_delay(timeout)

    def recv(self, file_io, crc_mode = 1, retry = 16, timeout = 15, delay = 1):
        return super(acomms_xmodem, self).recv(file_io, crc_mode, retry, timeout, delay)

    def send(self, file_io, retry = 16, timeout = 15, delay = 1, callback = None):
        return super(acomms_xmodem, self).send(file_io, retry, timeout, delay, callback)

    def __init__(self, micromodem, dest, rate = 1, timeout = 60, pad = '\x1a',
                 unified_log = None, log_path = None):
        super(acomms_xmodem, self).__init__(micromodem, dest, rate, timeout, pad, False, unified_log, log_path)


class acomms_ymodem(acomms_xymodem):
    def calculate_packet_delay(self, timeout = 30):
        super(acomms_ymodem, self).calculate_packet_delay(timeout)

    def __init__(self, micromodem, dest, rate = 1, timeout = 60, pad = '\x1a',
                 unified_log = None, log_path = None):
        super(acomms_ymodem, self).__init__(micromodem, dest, rate, timeout, pad, True, unified_log, log_path)

    def recv(self, file_io, crc_mode = 1, retry = 16, timeout = 15, delay = 1):
        return super(acomms_ymodem, self).recv(file_io, crc_mode, retry, timeout, delay)

    def send(self, file_io, retry = 16, timeout = 15, delay = 1, callback = None):
        return super(acomms_ymodem, self).send(file_io, retry, timeout, delay, callback)



class acomms_zmodem(object):
    def __init__(self, micromodem, dest,rate = 1, timeout=60,pad='\x1a',ymodem_enabled = True,unified_log=None,log_path=None, ):
        assert isinstance(micromodem, Micromodem), "micromodem object isn't a Micromodem: %r" & micromodem
        assert rate in range(0,6), "Invalid Rate: %d" & rate
        assert dest != micromodem.id, "Can't send file to self."
        self.pad = pad

        self.rate = Rates[rate]
        self.micromodem = micromodem
        self.micromodem.set_config('RXP',1)
        self.micromodem.set_config('MOD',1)
        #use CRC-8/CDMA2000 instead of CRC-16 CCITT Unreflected.
        self.calc_crc = crcmod.mkCrcFun(0x19b, rev=False, initCrc=0xff, xorOut=0x00)
        if unified_log is None:
            unified_log = UnifiedLog(log_path=log_path)
        self.log = unified_log.getLogger("xmodem.{0}".format(micromodem.name))
        self.target_id = dest
        self.timeout = timeout
        self.ymodem_mode = True

    SOH = '0001'
    STX = '0002'
    EOT = '0004'
    ACK = '0006'
    DLE = '0010'
    NAK = '0015'
    CAN = '0018'
    CRC = '0043'


from messageparams import CycleInfo, DrqParams, DataFrame, Ack, data_from_hexstring, hexstring_from_data
from cyclestats import CycleStats, TransmitStats
from modemlog import ModemLog
from timeutil import *
import commstate
from binascii import hexlify, unhexlify
import datetime
import sys
import traceback
import os

class MessageParser:
    def __init__(self, modem):
        self.modem = modem    
    
    # This method gets called when the object is called as a function, and
    # tries to find a matching attribute for that message type.
    # This is a pythonic(?) way of implementing a switch case.
    def parse(self, msg):
        try:
            func = getattr(self, msg['type'])
        except AttributeError, e:
            self.modem._daemon_log.warn('Unrecognized message: ' + str(msg['type']))
            func = None
        try:
            if func != None:
                return func(msg)
        except Exception, e:
            self.modem._daemon_log.error("Exception when parsing: " + str(sys.exc_info()[0]))
            traceback.print_exc()

    def GPRMC(self,msg):
        pass

    def PUBX(self,msg):
        pass

    def CACFG(self, msg):
        # Only worry about the parameters that matter...
        key = msg["params"][0]
        value = msg["params"][1]
        
        if key == "SRC": self.modem.id = int(value)
        if key == "ASD": self.modem.asd = bool(int(value))
        if key == "PCM": self.modem.pcm_on = bool(int(value))
        self.modem.config_data[key] = value
        
    def CACYC(self, msg):
        src = int(msg["params"][1])
        dest = int(msg["params"][2])
        rate = int(msg["params"][3])
        ack = int(msg["params"][4]) == 1
        num_frames = int(msg["params"][5])
        
        cycleinfo = CycleInfo(src, dest, rate, ack, num_frames)
        
        # Pass this to the comms state machine.
        self.modem.state.got_cacyc(cycleinfo)
        
    def CATXF(self, msg):
        self.modem.state.got_catxf()
        
    def CADRQ(self, msg):
        src = int(msg["params"][1])
        dest = int(msg["params"][2])
        ack = int(msg["params"][3]) == 1
        num_bytes = int(msg["params"][4])
        frame_num = int(msg["params"][5]) 
        
        drqparams = DrqParams(src, dest, ack, num_bytes, frame_num)
        
        self.modem.state.got_cadrq(drqparams)
        
    def CARXD(self, msg):
        src = int(msg["params"][0])
        dest = int(msg["params"][1])
        ack = int(msg["params"][2]) == 1
        frame_num = int(msg["params"][3])
        data = data_from_hexstring(msg["params"][4])
        
        dataframe = DataFrame(src, dest, ack, frame_num, data)
        
        self.modem.state.got_carx(dataframe)
        self.modem.on_rxframe(dataframe)
        
    def CAMSG(self, msg):
        # CAMSG sucks.  We need to parse it to figure out what's going on.
        # This doesn't account for all of the possible CAMSG messages.
        if msg["params"][0] == "BAD_CRC":
            self.modem.state.got_badcrc()
        elif msg["params"][0] == "PACKET_TIMEOUT":
            self.modem.state.got_packettimeout()
        else:
            try:
                msg_type = msg["params"][0]
                number = int(msg["params"][1])
                self.modem.state.got_camsg(msg_type,number)
            except ValueError:
                pass
        #TODO: Add PSK errors here
            
    def CAERR(self, msg):
        # Sigh.  This really shouldn't be used to signal data timeouts, but it is.
        # This doesn't account for most of the CAERR messages.
        if msg["params"][1] == "DATA_TIMEOUT":
            frame_num = msg["params"][2]
            self.modem.state.got_datatimeout(frame_num)
        else:
            hhmmss = msg["params"][0]
            module = msg["params"][1]
            err_num = int(msg["params"][2])
            message = msg["params"][3]
            self.modem.state.got_caerr(hhmmss,module,err_num,message)
    
    def CAREV(self, msg):
        '''Revision Message'''
        self.modem.state.got_carev(msg)
        
    def CATXP(self, msg):
        '''Start of Packet Transmission Acoustically'''
        pass
    
    def CAMPA(self,msg):
        '''Ping Received Acoustically'''
        src = int(msg["params"][0])
        dest = int(msg["params"][1])
        self.modem.state.got_campa(src,dest)
        pass

    def CADQF(self,msg):
        '''Data Quality Factor Message'''
        dqf= int(msg["params"][0])
        p = int(msg["params"][1])        
        self.modem.state.got_cadqf(dqf,p)
        pass
    
    def CARSP(self, msg):
        '''Echo of CCRSP command'''
        pass
    
    def CAXST(self, msg):
        '''Transmit Statistics message'''
        try:
            xst = TransmitStats.from_nmea_msg(msg)

            # Raise the event
            self.modem.on_xst(xst, msg)
        except Exception, ex:
            self.modem._daemon_log.error("Error parsing XST: " + str(sys.exc_info()[0]))
        pass
    
    def CARXP(self, msg):
        '''Probe received (RX pending)'''
        pass
    
    def CAMPC(self, msg):
        '''Ping command echo'''
        pass

    def CAMPR(self, msg):
        '''Ping command reply'''
        pass
    
    def CAMUC(self, msg):
        '''User minipacket command echo'''
        pass
        
    def CAMUR(self, msg):
        '''User minipacket reply'''
        pass

    def CAMUA(self, msg):
        '''User minipacket received accoustically'''
        pass
        
    def CATXD(self, msg):
        '''CCTXD echo'''
        pass
        
    def CAMSC(self, msg):
        '''Sleep command echo'''
        pass

    def CATMS(self, msg):
        '''Time set command echo '''
        pass

    def CATMG(self, msg):
        '''Time status '''
        pass

    def CARBR(self, msg):
        '''Log message retreival
        '''
        pass

    def CAHIB(self, msg):
        '''hibernate echo
        '''
        pass

    def CATDP(self, msg):
        ''' Transmit echo of new packet
        '''
        pass

    def CARDP(self, msg):
        ''' Received new packet
        '''
        pass

    def CAHBR(self, msg):
        '''Hibernate status
        '''
        pass

    def CAALQ(self, msg):
        '''API level query response
        '''
        pass

    def CACLK(self, msg):
        # $CACLK,yyyy,MM,dd,HH,mm,ss
        args = msg["params"]
        datestr = str(args[1]) + str(args[2]) + str(args[3]) + str(args[4]) + str(args[0]) + '.' + str(args[5])
        if self.modem.set_host_clock_flag == True:
            self.modem._daemon_log.warn("Setting host clock to: " + datestr)
            #TODO: This probably shouldn't be part of this module.
            os.system('/bin/date -u ' + datestr)
            self.modem.set_host_clock_flag = False
    
    def CACST(self, msg):

        try:
            cst = CycleStats.from_nmea_msg(msg)

            # Raise the event
            self.modem.on_cst(cst, msg)
        except Exception, ex:
            self.modem._daemon_log.error("Error parsing CST: " + str(sys.exc_info()[0]))
            raise

    def CATRC(self, msg):
        pass

    def CAACK(self, msg):
        src = int(msg["params"][0])
        dest = int(msg["params"][1])
        frame_num = int(msg["params"][2])
        ack = int(msg["params"][3]) == 1

        ack = Ack(src, dest, ack, frame_num)
        self.modem.on_ack(ack,msg)

    def CATOA(self, msg):
        pass

    def CABBD(self,msg):
        pass

    def CABDD(self, msg):
        pass

    def CATDP(self,msg):
        #errflag = int(msg["params"][0])
        #uniqueID = int(msg["params"][1])
        #dest = int(msg["params"][2])
        #rate = int(msg["params"][3])
        #ack = int(msg["params"][4]) == 1
        #reserved = int(msg["params"][5])

        #mfdata = (msg(["params"][6])).split(';')
        #dfdata = (msg(["params"][7])).split(';')
        pass



    def CARDP(self,msg):
        #src = int(msg["params"][0])
        #dest = int(msg["params"][1])
        #rate = int(msg["params"][2])
        #ack = int(msg["params"][3]) == 1
        #reserved = int(msg["params"][4])

        #mfdata = (msg(["params"][5])).split(';')
        #mf_crc_check = mfdata[0]
        #mf_nbytes = mfdata[1]
        #mf_data = data_from_hexstring(mfdata[2])

        #dfdata = (msg(["params"][6])).split(';')
        #df_crc_check = dfdata[0]
        #df_nbytes = dfdata[1]
        #df_data = data_from_hexstring(dfdata[2])
        pass

    def CAALQ(self,msg):
        app_name = msg["params"][0]
        nmea_api_level = int(msg["params"][1])
        self.modem._daemon_log.info("Modem API Information: Application Name: {0} API Level: {1}".format(app_name,nmea_api_level))

		
    def CAFDR(self,msg):
        src = int(msg["params"][0])
        dest = int(msg["params"][1])
        rate = int(msg["params"][2])
        ack = int(msg["params"][3]) == 1
        nbytes  = int(msg["params"][4])


    def CATMG(self,msg):
        dt = convert_to_datetime(msg["params"][0])
        clksrc = msg["params"][1]
        pps_source = msg["params"][2]

        self.modem._daemon_log.info("Modem Date and Time Message:{0}\tClock Source:{1}\tPPS Source:{2}".format(dt,clksrc.replace('_',' '),pps_source.replace('_',' ')))

    def CATMQ(self,msg):
        dt = convert_to_datetime(msg["params"][0])
        clksrc = msg["params"][1]
        pps_source = msg["params"][2]

        self.modem._daemon_log.info("Modem Date and Time Query Response:{0}\tClock Source:{1}\tPPS Source:{2}".format(dt,clksrc.replace('_',' '),pps_source.replace('_',' ')))

    def CAPAS(self,msg):
        passthrough_msg = msg["params"][0]

        self.modem._daemon_log.info("Pass Through Message Received: {0}".format(passthrough_msg))

    def CAPST(self,msg):
        pass

    def CAHIB(self,msg):
        hibernate_cause = int(msg["params"][0])
        hibernate_time = convert_to_datetime(msg["params"][1])
        wake_cause = int(msg["params"][2])
        wake_time = convert_to_datetime(msg["params"][3])
        self.modem._daemon_log.info("Modem Hibernate Ack: Hibernate({0},{1}), Wake({2},{3})".format(hibernate_cause,hibernate_time,wake_cause,wake_time))
        pass

    def CAMEC(self,msg):
        pass

    def CAHBR(self,msg):
        wake_time = convert_to_datetime(msg["params"][0])
        self.modem._daemon_log.info("Modem Hibernate Start: Wake@{0}".format(wake_time))

    def CATMS(self,msg):
        dt = None
        timed_out = int(msg["params"][0])
        if msg["params"][1] is not None:
            dt = convert_to_datetime(msg["params"][1])
        self.modem._daemon_log.info("Modem Set Clock Response: Timed Out:{0} Time Set To:{1}".format(timed_out,dt))

    def CARBS(self,msg):
        pass
        #try:
        #    log = ModemLog.from_nmea_msg(msg)

            # Raise the event
        #    self.modem.on_modem_log(log, msg)
        #except Exception, ex:
        #    self.modem._daemon_log.error("Error parsing Retrieved Modem Log Message: " + str(sys.exc_info()[0]))

    def CARBR(self,msg):
        pass
        #try:
        #    log = ModemLog.from_nmea_msg(msg)

            # Raise the event
        #    self.modem.on_modem_log(log, msg)
        #except Exception, ex:
        #    self.modem._daemon_log.error("Error parsing Retrieved Modem Log Message: " + str(sys.exc_info()[0]))

    def CAMIZ(self,msg):
        pass

    def SNTTA(self,msg):
        pass

    def SNMFD(self,msg):
        pass

    def SNUTX(self,msg):
        pass

    def UPMFWA(self,msg):
        slot = int(msg["params"][0])
        data_loc = int(msg["params"][1])
        total_data_size = int(msg["params"][2])
        sha1_hash = msg["params"][3]
        self.modem._daemon_log.info("Modem Starting FW Update Process: Slot {}, {} bytes, SHA1 HASH = ({}).".format(slot,total_data_size,sha1_hash))
        pass

    def UPDATA(self,msg):
        nbytes_received = int(msg["params"][0])
        self.modem._daemon_log.info("Modem Received {} Bytes of Firmware File Upload.".format(nbytes_received))
        pass

    def UPDONE(self,msg):
        update_msg = msg["params"][0]
        self.modem._daemon_log.info("Modem FW Update Done: {}".format(update_msg))
        pass

    def UPERR(self,msg):
        errno = int(msg["params"][0])
        err_msg = msg["params"][1]
        self.modem._daemon_log.error("Modem FW Update Errored ({}): {}".format(errno,err_msg))
        pass



import abc

class ModemConnection(object):
    ''' Abstract class for connecting to a Micromodem via some protocol (such as serial, TCP, etc).
    '''

    metaclass__ = abc.ABCMeta

    @abc.abstractproperty
    def is_connected(self):
        pass

    @abc.abstractproperty
    def can_change_baudrate(self):
        pass

    @abc.abstractmethod
    def change_baudrate(self,baudrate):
        pass

    @abc.abstractmethod
    def _listen(self):
        pass

    @abc.abstractmethod
    def close(self):
        pass

    @abc.abstractmethod
    def write(self,data):
        pass


__author__ = 'andrew'

from modem_connection import ModemConnection
import asyncore
from socket import AF_INET, SOCK_STREAM
from threading import Thread

class TcpConnection(ModemConnection, asyncore.dispatcher_with_send):

    def __init__(self,modem, ipaddr,port):
        asyncore.dispatcher_with_send.__init__(self)
        self.create_socket(AF_INET, SOCK_STREAM)
        self.connect((ipaddr,port))
        self.io_file = self.socket.makefile()
        self.modem = modem

        self._thread = Thread(target=self._listen)
        self._thread.setDaemon(True)
        self._thread.start()

    def _listen(self):
        asyncore.loop()


    def is_connected(self):
        return True

    def write(self, data):
        self.send(data)

    def can_change_baudrate(self):
        return False

    def change_baudrate(self, baudrate):
        return 9600

    def handle_close(self):
        self.io_file.close()
        asyncore.dispatcher_with_send.close(self)

    def writable(self):
        return self.modem._message_waiting()

    def handle_read(self):
        msg = self.io_file.readline()
        self.modem._process_incoming_nmea(msg)


    def handle_write(self):
        self.modem._process_outgoing_nmea()


from binascii import hexlify, unhexlify
import exceptions

def data_from_hexstring(hexstring):
    databytes = bytearray()
    try:
        databytes.extend([ord(c) for c in unhexlify(hexstring)])
    #Catch Odd-length String Error
    except exceptions.TypeError:
        pass
    
    return databytes

def hexstring_from_data(databytes):
    return hexlify(bytes(databytes))

class CycleInfo(object):
    def __init__(self, src, dest, rate_num, ack=False, num_frames=None):
        self.src = int(src)
        self.dest = int(dest)
        self.rate_num = int(rate_num)
        self.ack = bool(ack)
        
        if num_frames == None:
            self.num_frames = Rates[rate_num].numframes
        else:
            self.num_frames = int(num_frames)
        
    # This allows us to see if two cycleinfo objects match
    def __eq__(self, other):
        return self.__dict__ == other.__dict__
        
class DrqParams(object):
    def __repr__(self):
        return "SRC: {} DST: {} ACK: {} NUMBYTES: {} FRAME#: {}".format(self.src,self.dest,self.ack,self.num_bytes,self.frame_num)

    def __init__(self, src, dest, ack, num_bytes, frame_num):
        self.src = int(src)
        self.dest = int(dest)
        self.ack = bool(ack)
        self.num_bytes = int(num_bytes)
        self.frame_num = int(frame_num)
        
class Ack(object):
    def __repr__(self):
        return "SRC: {} DST: {} FRAME#: {}".format(self.src,self.dest,self.frame_num)

    def __init__(self, src, dest, ack, frame_num):
        self.src = int(src)
        self.dest = int(dest)
        self.ack = bool(ack)
        self.frame_num = int(frame_num)



class DataFrame(object):
    def __init__(self, src, dest, ack, frame_num, data, bad_crc=False):
        self.bad_crc = bad_crc
        self.src = src
        self.dest = dest
        self.ack = ack
        self.frame_num = frame_num
        if data:
            self.data = bytearray(data)
        else:
            self.data = None

    def __repr__(self):
        return "SRC: {} DST: {} ACK: {} FRAME#: {} BAD_CRC: {} DATA: {}".format(self.src,self.dest,self.ack,self.frame_num, self.bad_crc, repr(self.data))
        
class Packet(object):
    def __init__(self, cycleinfo, frames=None):
        self.cycleinfo = cycleinfo
        
        if frames != None:
            self.frames = frames
        else:
            self.frames = []
            
    def append_framedata(self, framedata):
        #TODO: Make sure we have room for another frame, and that the data fits in the frame.
        newframe = DataFrame(self.cycleinfo.src, self.cycleinfo.dest, self.cycleinfo.ack, 
                             (len(self.frames) + 1), framedata)
        self.frames.append(newframe)
            
        
class PacketRate(object):
    def __init__(self, name, number, framesize, numframes):
        self.name = name
        self.number = number
        self.framesize = framesize
        self.numframes = numframes
    
    def getpacketsize(self):
        return self.framesize * self.numframes
        
    maxpacketsize = property(getpacketsize)

Rates = {0:PacketRate('FH-FSK', 0, 32, 1),
         1:PacketRate('BCH 128:8', 1, 64, 3),
         2:PacketRate('DSS 1/15 (64B frames)', 2, 64, 3),
         3:PacketRate('DSS 1/7', 3, 256, 2),
         4:PacketRate('BCH 64:10', 4, 256, 2),
         5:PacketRate('Hamming 14:9', 5, 256, 8),
         6:PacketRate('DSS 1/15 (32B frames)', 6, 32, 6)}

FDPRates = {1:PacketRate('BCH 128:8', 1, 100, 1),
            3:PacketRate('BCH 64:10', 3, 100, 1)}


    


#!/usr/bin/env python

import os
try:
    from ez_setup import use_setuptools
    use_setuptools()
except ImportError:
    pass

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name='acomms',
    version='0.7.0',
    author='Eric Gallimore, Andrew Beal',
    author_email='pyacomms@whoi.edu',
    packages=['acomms', 'acomms.modem_connections', 'bin',],
    url='http://acomms.whoi.edu/',
    license='LGPLv3+',
    description='WHOI Micromodem Interface Library and Tools',
    long_description=read('README.txt'),
    install_requires=[
        "bitstring >= 3.0.0",
        "pyserial >= 2.6",
        "isodate >= 0.4.9",
        "python-dateutil >= 2.1",
        "timer2 >= 0.1.0",
        "enum >= 0.4.4",
        "apscheduler>=2.1.1",
        "crcmod>=1.7"
    ],
    py_modules=['ez_setup'],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)",
        "Programming Language :: Python :: 2.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Scientific/Engineering",
    ],
)




from micromodem import *
from commstate import *
from cyclestats import *
from messageparams import *
from messageparser import *
from unifiedlog import UnifiedLog
from modemlog import *
from acomms_xmodem import *
